#!/usr/bin/env python3
# coding: utf-8
# -*- coding: utf-8 -*-
# A Fediverse (decentralized social network, for instance using Mastodon) bot

from mastodon import StreamListener
from lxml import html
from logging.handlers import RotatingFileHandler
from pprint import pprint
from utils.config import get_parameter, init_log, init_mastodon

import requests, os, random, sys, time, json, logging, argparse, re

config_file        = "config.txt"
secrets_filepath   = "secrets/secrets.txt"
log_filepath       = "activity.log"
blacklist_filepath = "blacklist.json"
log = init_log(log_filepath)
mastodon = init_mastodon(config_file, secrets_filepath)

blacklist_file = open(blacklist_filepath,'r')
BLACKLIST = json.loads(blacklist_file.read())
blacklist_file.close()


def post_img(mastodon, text, log, config):
    img_path = get_parameter("img_path", config)
    file = random.choice(os.listdir(img_path+"/")) 
    image_byte = open(img_path+"/"+file, "rb").read()

    file, ext = os.path.splitext(file)
    mime_dict = {'.jpg': 'image/jpeg', '.jpe': 'image/jpeg', '.jpeg': 'image/jpeg', '.png': 'image/png', '.gif': 'image/gif'}
    
    mime = mime_dict[str.lower(ext)]
    #try:

    #except KeyError:
    #    mime = None;
    #    pass

    media_dict = mastodon.media_post(image_byte, mime)
    return media_dict;
    
def cleanhtml(raw_html):
    cleanr = re.compile('<.*?>')
    cleantext = re.sub(cleanr, '', raw_html)
    return cleantext

class BotListener(StreamListener):

	# use only notification
	def on_notification(self, notification):

			# catch only mention in notification
			if notification['type'] == 'mention':
				log.debug("Got a mention")
				if notification["account"]["bot"] == False:
					sender = notification['account']['acct'] # Get sender name
					if sender in BLACKLIST:
						log.info("Service refused to %s" % sender)
						return
					sender_hour_filename = "limiter/hour/" + sender; # Forge file for limiter
					sender_minute_filename = "limiter/minute/" + sender; # Forge file for limiter
					
					if os.path.isfile(sender_hour_filename): # Check if file exist
						log.debug("Sender file exist")
						statbuf = os.stat(sender_hour_filename) 
						last_edit = int(statbuf.st_mtime)
						ts = int(time.time())
						if ts - last_edit > 3599: # check if file is modified 1 hour after last edition
							log.debug("file is too old")
							f = open(sender_hour_filename,'w')
							f.write(str(1)) # reset counter
							f.close()
							can_continue = True
						else: 
							log.debug("file is young")
							f = open(sender_hour_filename,'r+')
							limit = int(get_parameter("limit_hour", config_file))
							number_of_mention = int(f.read())
							if number_of_mention < limit: # limit of mention per hour is limit_hour
								log.debug("Sender have less of limit requests")
								f.seek(0)
								f.write(str(number_of_mention + 1))
								can_continue = True
							else:
								log.debug("Sender have more of limit requests")
								can_continue = False # if number of mention is for, user can't receive anything
							f.close()
					else: # File not exist, create it and initialise it
						log.debug("Sender file not exist")
						f = open(sender_hour_filename,"w+")
						f.write(str(1))
						f.close()
						can_continue = True
					
					if can_continue:
						if os.path.isfile(sender_minute_filename): # Check if file exist
							log.debug("Sender file exist")
							statbuf = os.stat(sender_minute_filename) 
							last_edit = int(statbuf.st_mtime)
							ts = int(time.time())
							if ts - last_edit > 59: # check if file is modified 1 minute after last edition
								log.debug("file is too old")
								f = open(sender_minute_filename,'w')
								f.write(str(1)) # reset counter
								f.close()
								can_continue = True
							else: 
								log.debug("file is young")
								f = open(sender_minute_filename,'r+')
								limit = int(get_parameter("limit", config_file))
								number_of_mention = int(f.read())
								if number_of_mention < limit: # limit of mention per minute is 4
									log.debug("Sender have less of limit requests")
									f.seek(0)
									f.write(str(number_of_mention + 1))
									can_continue = True
								else:
									log.debug("Sender have more of limit requests")
									can_continue = False # if number of mention is for, user can't receive anything
									file = open(sender_hour_filename,'r+')
									number_of_mention = int(file.read())
									file.seek(0)
									file.write(str(number_of_mention - 1))
									file.close()
								f.close()
						else: # File not exist, create it and initialise it
							log.debug("Sender file not exist")
							f = open(sender_minute_filename,"w+")
							f.write(str(1))
							f.close()
							can_continue = True

					if can_continue:
						id = notification['status']['id']
						visibility = notification['status']['visibility']
						if visibility == 'public':
							visibility = 'unlisted'
						mentions = notification['status']['mentions']
						text = "@" + notification['status']["account"]["acct"] + " "
						for mention in mentions:
							if mention["acct"] != get_parameter("name_bot", config_file):
								text = text + "@" + mention["acct"] + " "
			    		
						if get_parameter("sensitive", config_file) == "yes":
							sensitive = True
						else:
							sensitive = False
						media_dict = post_img(mastodon, get_parameter("default_text", config_file), log, config_file)
						mastodon.status_post(text, None, media_ids=[media_dict], sensitive=sensitive, visibility=visibility, spoiler_text=get_parameter("spoiler_text", config_file))
					else: 
						log.debug("no picture send :(")
						pass
			else:
				log.debug("Nevermind")


def main():
    
    parser = argparse.ArgumentParser(description='Choose between image or streaming')
    parser.add_argument("-i", "--img", action='store_true', help="post image")
    parser.add_argument("-s", "--stream", action="store_true", help="stream user profile")
    args = parser.parse_args()
    if args.img:
        media_dict = post_img(mastodon, get_parameter("default_text", config_file), log, config_file)
        if get_parameter("sensitive", config_file) == "yes":
        	sensitive = True
        else:
        	sensitive = False
        mastodon.status_post(get_parameter("default_text", config_file), None, media_ids=[media_dict], sensitive=sensitive, visibility='public', spoiler_text=get_parameter("spoiler_text", config_file))
        sys.exit()
    elif args.stream:
        stream = BotListener();
        while True:
            try:
                log.info("Start listening...")
                mastodon.stream_user(stream)
            except Exception as error:
                log.warning('General exception caught: ' + str(error))
                time.sleep(0.5)
    else:
        print("Require an argument. Use --help to display help")

main()

